import { parseFunction } from '../shared/helpers/parse-function'
import { Doc, DocType } from '../shared/types/doc-types'

const FRAMES_WITH_FRAMES = [
  'MessageFrame',
  'Frame',
  'FrameScriptObject',
  'ScrollFrame',
  'FogOfWarFrame',
  'UnitPositionFrame',
  'ArchaeologyDigSiteFrame'
]

const ANIMS_WITH_ANIM = [
  'AnimAlpha',
  'AnimFlipBook',
  'AnimPath',
  'AnimRotation',
  'AnimTextureCoordTranslation',
  'AnimTranslation',
  'AnimScale',
  'AnimVertexColor'
]

const SHOULD_IGNORE = ['Frame']

export const getParsedScriptObjects = (data: Doc[]) => {
  const PARSED_SCRIPT_OBJECTS: string[] = []

  for (const segment of data) {
    if (segment.Type === DocType.ScriptObject && segment.Name) {
      // For some reason the cooldown frame is documented twice
      if (segment.Functions?.length && segment.Name !== 'CooldownFrameAPI') {
        const PARSED_FUNCTIONS: string[] = []

        let cleanName = segment.Name.replace(/(API|Simple)/gi, '')

        if (!FRAMES_WITH_FRAMES.includes(cleanName)) {
          cleanName = cleanName.replace('Frame', '')
        }

        if (cleanName.includes('Anim') && ANIMS_WITH_ANIM.includes(cleanName)) {
          cleanName = cleanName.replace('Anim', '')
        }

        if (cleanName === 'ModelSceneFrameActorBase') cleanName = 'ModelSceneActorBase'
        if (cleanName === 'ModelSceneFrameActor') cleanName = 'ModelSceneActor'
        if (cleanName === 'ModelSceneFrame') cleanName = 'ModelScene'
        if (cleanName === 'AnimGroup') cleanName = 'AnimationGroup'
        if (cleanName === 'Anim') cleanName = 'Animation'
        if (cleanName === 'Object') cleanName = 'ScriptObject'

        if (!SHOULD_IGNORE.includes(cleanName)) {
          for (const func of segment.Functions) {
            PARSED_FUNCTIONS.push(parseFunction(func, cleanName))
          }

          PARSED_SCRIPT_OBJECTS.push(`
          interface ${cleanName} {
            ${PARSED_FUNCTIONS.join('\n')}
          }
        `)
        }
      }
    }
  }

  console.log(`Parsed ${PARSED_SCRIPT_OBJECTS.length} ScriptObjects`)

  return PARSED_SCRIPT_OBJECTS
}
