import { parseField } from '../shared/helpers/parse-field'
import { Doc, TableType } from '../shared/types/doc-types'
import { CallBackTable } from '../shared/types/doc-types'

const parseCallback = (callback: CallBackTable) => {
  const PARSED_FIELDS: string[] = []

  if (callback.Arguments) {
    for (const field of callback.Arguments) {
      PARSED_FIELDS.push(parseField(field))
    }
  }

  return `
    type ${callback.Name} = (${PARSED_FIELDS.join(',')}) => void
  `
}

export const getParsedCallbacks = (data: Doc[]) => {
  const CALLBACKS: string[] = []
  const PARSED_CALLBACKS: string[] = []

  for (const segment of data) {
    if (segment.Tables.length > 0) {
      for (const table of segment.Tables) {
        if (
          table.Type === TableType.CallbackType &&
          table.Name !== 'PendingPingOffScreenCallback' &&
          !PARSED_CALLBACKS.includes(table.Name)
        ) {
          CALLBACKS.push(parseCallback(table))
          PARSED_CALLBACKS.push(table.Name)
        }
      }
    }
  }

  // Save Struct declarations
  console.log(`Parsed ${CALLBACKS.length} Callbacks`)

  return CALLBACKS
}
