import { Doc, TableType } from '../shared/types/doc-types'
import { ConstantTable } from '../shared/types/doc-types'

const parseConstant = (table: ConstantTable) => {
  const realValues = table.Values.filter((value) => !!value.Value)

  if (!realValues.length) {
    return
  }

  return `
    const ${table.Name}: { ${table.Values.map(
      (value) => `${value.Name}: ${value.Value}`
    )} }
  `
}

export const getParsedConstants = (data: Doc[]) => {
  const CONSTANTS: string[] = []
  const PARSED_CONSTANTS: string[] = []

  for (const segment of data) {
    if (segment.Tables.length > 0) {
      for (const table of segment.Tables) {
        if (
          table.Type === TableType.Constants &&
          !PARSED_CONSTANTS.includes(table.Name)
        ) {
          const constant = parseConstant(table)

          if (constant) {
            CONSTANTS.push(constant)
            PARSED_CONSTANTS.push(table.Name)
          }
        }
      }
    }
  }
  console.log(`Parsed ${CONSTANTS.length} Constants`)

  return CONSTANTS
}
