import { Doc, EnumTable, TableType } from '../shared/types/doc-types'

const parseEnum = (value: EnumTable) => {
  return `enum ${value.Name} { ${value.Fields.map(
    (field) => `${field.Name} = ${field.EnumValue}`
  ).join(',')} }`
}

export const getParsedEnums = (data: Doc[]) => {
  const ENUMS: string[] = []
  // The UnitSex enum has a name conflict with a global function so we skip it.
  const PARSED_ENUMS: string[] = ['UnitSex']

  for (const segment of data) {
    if (segment.Tables.length > 0) {
      for (const table of segment.Tables) {
        if (table.Type === TableType.Enumeration && !PARSED_ENUMS.includes(table.Name)) {
          ENUMS.push(parseEnum(table))
          PARSED_ENUMS.push(table.Name)
        }
      }
    }
  }

  console.log(`Parsed ${ENUMS.length} Enums`)

  return ENUMS
}
