import { onlyUnique } from '../shared/helpers/only-unique'
import { parseField } from '../shared/helpers/parse-field'
import { Doc } from '../shared/types/doc-types'

export const getParsedEventData = (data: Doc[]) => {
  const EVENT_KEYS: string[] = []
  const PARSED_EVENTS: string[] = []

  for (const segment of data) {
    if (segment.Events?.length) {
      for (const event of segment.Events) {
        EVENT_KEYS.push(event.LiteralName)
        PARSED_EVENTS.push(`
          ${event.LiteralName}: ${
            !event.Payload
              ? undefined
              : `LuaMultiReturn<[${event.Payload.map((payload) =>
                  parseField(payload)
                ).join(',')}]>`
          }
        `)
      }
    }
  }

  const UNIQUE_KEYS = EVENT_KEYS.filter(onlyUnique)

  console.log(`Saving ${UNIQUE_KEYS.length} Events`)

  return `
    interface WowEventPayload {
      ${PARSED_EVENTS.join('\n')}
    }

    type WoWEventKey = keyof WowEventPayload
  `
}
