import { parseField } from '../shared/helpers/parse-field'
import { Doc, TableType } from '../shared/types/doc-types'
import { StructureTable } from '../shared/types/doc-types'

export const parseStruct = (data: StructureTable) => {
  const PARSED_TYPES: string[] = []
  for (const field of data.Fields) {
    const parsedField = parseField(field)

    PARSED_TYPES.push(parsedField)
  }
  if (PARSED_TYPES.length !== 0) {
    return `
      interface ${data.Name} {
        ${PARSED_TYPES.join(',\n')}
      }
    `
  }
  return undefined
}

export const getParsedStructs = (data: Doc[]) => {
  const STRUCTURES: string[] = []
  const PARSED_STRUCTURES: string[] = []

  for (const segment of data) {
    if (segment.Tables.length > 0) {
      for (const table of segment.Tables) {
        if (
          table.Type === TableType.Structure &&
          !PARSED_STRUCTURES.includes(table.Name)
        ) {
          const struct = parseStruct(table)

          if (struct) {
            STRUCTURES.push(struct)
            PARSED_STRUCTURES.push(table.Name)
          }
        }
      }
    }
  }

  // Save Struct declarations
  console.log(`Parsed ${STRUCTURES.length} Structures`)

  return STRUCTURES
}
