import { parseFunction } from '../shared/helpers/parse-function'
import { Doc, DocType } from '../shared/types/doc-types'

const SYSTEMS_TO_SKIP = ['C_Timer']

export const getParsedSystems = (data: Doc[]) => {
  const PARSED_SYSTEMS: string[] = [
    `function CanInspect(token: UnitToken, showError?: boolean): boolean`,
    `function GetSpecializationRoleByID(specId: number): "DAMAGER" | "TANK" | "HEALER" | undefined`,
    `function IsPlayerSpell(spellId: number): boolean`,
    `function GetSpellCooldown(spellId: number): LuaMultiReturn<[number, number, number, number]>`,
    `function GetShapeshiftFormID(): number`,
    `function UnitHasIncomingResurrection(token: string): boolean`,
    `function CreateFrame<T extends FrameType>(type: T, name?: string, parent?: FrameGeneric<T>, template?: string, id?: number): FrameGeneric<T>`
  ]

  const systemSegments = data.filter((segment) => segment.Type === DocType.System)

  for (const segment of systemSegments) {
    const PARSED_FUNCTIONS: string[] = []

    if (!segment.Namespace || !SYSTEMS_TO_SKIP.includes(segment.Namespace)) {
      if (segment.Functions) {
        for (const func of segment.Functions) {
          PARSED_FUNCTIONS.push(parseFunction(func))
        }

        if (PARSED_FUNCTIONS.length > 0) {
          if (segment.Namespace) {
            PARSED_SYSTEMS.push(`
            namespace ${segment.Namespace} {
              ${PARSED_FUNCTIONS.join('\n')}
            }
          `)
          } else {
            PARSED_SYSTEMS.push(PARSED_FUNCTIONS.join('\n'))
          }
        }
      }
    }
  }

  console.log(`Parsed ${PARSED_SYSTEMS.length} Systems`)
  return PARSED_SYSTEMS
}
