import { Doc, Field, TableType } from '../types/doc-types'
import { WOW_TYPE_KEYS, WowTypeKeyAliases } from '../types/wow-type-aliases'

const data = require('../../../lua-docs/docs/docs.json') as Doc[]

const NAMES_TO_SKIP = ['PendingPingOffScreenCallback', 'ScriptObject', 'UnitSex']
const SAFE_TYPES = [TableType.Structure, TableType.Enumeration, TableType.CallbackType]

export const parseField = (field: Field) => {
  const typeName = field.Type

  if (field.Name === 'in') {
    field.Name = 'inValue'
  }

  const correspondingTypeDef = data
    .filter((segment) => segment.Tables.length !== 0)
    .flatMap((segment) => segment.Tables)
    .find((table) => table.Name === typeName)

  // If we found a type for this name in the data, and its an Enum, Struct, or Callback
  // then we can assume its safe to declare without aliasing.
  if (
    correspondingTypeDef &&
    !NAMES_TO_SKIP.includes(typeName) &&
    SAFE_TYPES.includes(correspondingTypeDef.Type)
  ) {
    return `${field.Name}: ${typeName}${field.Nilable ? ' | undefined' : ''}`
  }

  if (!WOW_TYPE_KEYS.includes(field.Type)) {
    throw new Error(`Encountered unknown string type: "${field.Type}"`)
  }

  let typeAlias = WowTypeKeyAliases[field.Type]

  // The 'table' type requires a bit of special parsing.
  if (typeof typeAlias === 'function') {
    typeAlias = typeAlias(field)
  }

  return `${field.Name}: ${typeAlias}${field.Nilable ? ' | undefined' : ''}`
}
