import { WowFunction } from '../types/doc-types'
import { parseField } from './parse-field'

const getReturnString = (toReturn: string[]) => {
  if (!toReturn.length) return 'void'
  const returnString = toReturn.join(',')
  if (toReturn.length > 1) {
    return `LuaMultiReturn<[${returnString}]>`
  }

  const splitReturn = returnString.split(':')

  return splitReturn[1]
}

export const parseFunction = (func: WowFunction, fieldParent?: string) => {
  const ARGUMENT_FIELDS: string[] = []
  const RETURN_FIELDS: string[] = []

  if (func.Name === 'SetScript' && fieldParent === 'ScriptRegion') {
    return ''
  }

  if (func.Arguments?.length) {
    for (const arg of func.Arguments) {
      ARGUMENT_FIELDS.push(parseField(arg))
    }
  }

  if (func.Returns?.length) {
    for (const returnField of func.Returns) {
      RETURN_FIELDS.push(parseField(returnField))
    }
  }

  return `
    ${fieldParent ? '' : 'function '}${func.Name}(${ARGUMENT_FIELDS.join(
      ','
    )}): ${getReturnString(RETURN_FIELDS)}
  `
}
