import { WowTypeKeys } from './wow-type-aliases'

export enum TableType {
  Enumeration = 'Enumeration',
  Structure = 'Structure',
  Constants = 'Constants',
  CallbackType = 'CallbackType'
}

export enum DocType {
  System = 'System',
  ScriptObject = 'ScriptObject'
}

export const VALID_TABLE_TYPES = [
  TableType.Enumeration,
  TableType.Structure,
  TableType.Constants,
  TableType.CallbackType
]

export interface Field {
  Name: string
  Type: WowTypeKeys
  Nilable: boolean
  InnerType: WowTypeKeys
  Mixin?: string
}

export interface CallBackTable {
  Name: string
  Type: TableType.CallbackType
  Arguments?: Field[]
}

export interface EnumTable {
  MinValue: number
  MaxValue: number
  NumValues: number
  Name: string
  Type: TableType.Enumeration
  Fields: {
    Name: string
    Type: string
    EnumValue: number
  }[]
}

export interface ConstantTable {
  Name: string
  Type: TableType.Constants
  Values: {
    Name: string
    Type: string
    Value?: unknown
  }[]
}

export interface StructureTable {
  Name: string
  Type: TableType.Structure
  Fields: Field[]
}

export interface WoWEvent {
  Type: 'Event'
  LiteralName: string
  Name: string
  Payload?: Field[]
}

export interface WowFunction {
  Returns?: Field[]
  Arguments?: Field[]
  Type: 'Function'
  Name: string
  Documentation?: string[]
}

export interface Doc {
  Tables: (CallBackTable | EnumTable | ConstantTable | StructureTable)[]
  Events?: WoWEvent[]
  Type?: DocType
  Name?: string
  Functions?: WowFunction[]
  Namespace?: string
}
