import { Field } from './doc-types'

/**
 * The wow docs contain a lot of types that dont neatly point to their actual type (e.g "fileID" instead of "number"),
 * or in some cases point to a type that we've moved elsewhere in the type heirarchy.
 * For this reason we have to take these type names, and map them to one usable by us
 */

export type WowTypeKeys = (typeof WOW_TYPE_KEYS)[number]

export const WowTypeKeyAliases: {
  [key: string]: string | ((field: Field) => string)
} = {
  string: 'string',
  number: 'number',
  BigInteger: 'number',
  FileAsset: 'string',
  ArtifactTiers: 'string',
  ConnectionIptype: 'string',
  ModelAsset: 'number',
  GameMode: 'string',
  JustifyVertical: 'number',
  GameRule: 'number',
  SpellIdentifier: 'number',
  ItemCreationContext: 'number',
  Vocalerrorsounds: 'number',
  UnitSex: 'number',
  mouseButton: 'unknown',
  JustifyHorizontal: 'string',
  normalizedValue: 'number',
  stringView: 'string',
  PendingPingOffScreenCallback: '(() => void)',
  UiMapPoint: 'UiMapPoint',
  LuaValueVariant: '{}',
  ReportInfo: 'ReportInfoMixin',
  ItemSoundType: 'ItemSoundType',
  SimpleControlPoint: 'unknown',
  size: 'number',
  ItemInfo: 'string | number',
  ScriptObject: 'ScriptObject',
  FilterMode: 'FilterMode',
  SimpleMaskTexture: 'ScriptObject',
  StatusBarFillStyle: 'StatusBarFillStyle',
  Orientation: 'Orientation',
  uiAddon: 'string | number',
  SimpleLine: 'Line',
  InsertMode: 'InsertMode',
  SimplePathAnim: 'ScriptObject',
  BlendMode: 'BlendMode',
  SimpleFontString: 'FontString',
  PlayerLocation: 'PlayerLocationMixin',
  SimpleWindow: 'unknown',
  SimpleFont: 'FontInfo',
  SimpleButtonStateToken: 'SimpleButtonStateToken',
  LoopType: 'LoopType',
  TBFFlags: 'FontFlags[]',
  SimpleAnim: 'Animation',
  CurveType: 'string',
  SingleColorValue: 'number',
  IDOrLink: 'string | number',
  luaFunction: '(() => void)',
  uiRect: '[number, number, number, number]',
  TextureAssetDisk: 'string',
  TextureAsset: 'string',
  SmoothingType: 'string',
  uiFontHeight: 'number',
  CScriptObject: 'ScriptObject',
  SimpleAnimGroup: 'AnimationGroup',
  SimpleTexture: 'Texture',
  DrawLayer: 'DrawLayer',
  ModelSceneFrameActor: 'ModelSceneActor',
  ItemTransmogInfo: 'ItemTransmogInfo',
  TooltipComparisonItem: 'TooltipComparisonItem',
  AnimationDataEnum: 'number',
  kstringLfgListChat: 'string',
  TransmogLocation: 'TransmogLocationMixin',
  NamePlateFrame: 'NamePlateFrame',
  UnitToken: 'UnitToken',
  WeeklyRewardItemDBID: 'string',
  CachedRewardType: 'CachedRewardType',
  WeeklyRewardChestThresholdType: 'WeeklyRewardChestThresholdType',
  RecruitAcceptanceID: 'string',
  kstringLfgListSearch: 'string',
  kstringLfgListApplicant: 'string',
  GarrisonFollower: 'number',
  colorRGBA: 'ColorMixin',
  kstringClubMessage: 'string',
  CalendarEventID: 'string',
  bool: 'boolean',
  vector2: 'Vector2DMixin',
  ClubId: 'string',
  EmptiableItemLocation: 'ItemLocationMixin',
  AzeriteItemLocation: 'ItemLocationMixin',
  luaIndex: 'number',
  NotificationDbId: 'string',
  fileID: 'number',
  cstring: 'string',
  ClubStreamId: 'string',
  ClubInvitationId: 'string',
  textureAtlas: 'string',
  table: (field: Field) => {
    if (field.InnerType === 'table') {
      throw new Error('String type error: Table of a table.')
    }
    return `${WowTypeKeyAliases[field.InnerType]}[]`
  },
  colorRGB: 'ColorMixin',
  BigUInteger: 'number',
  WOWMONEY: 'number',
  TooltipData: 'TooltipData',
  time_t: 'number',
  AuraData: 'AuraData',
  ModelSceneFrame: 'ModelScene',
  TransmogPendingInfo: 'TransmogLocationMixin',
  WOWGUID: 'string',
  textureKit: 'string',
  ItemLocation: 'ItemLocationMixin',
  vector3: 'Vector3DMixin',
  HTMLTextType: 'unknown',
  TBFStyleFlags: 'unknown',
  FramePoint: 'string',
  ScriptRegion: 'unknown',
  uiUnit: 'number',
  SimpleFrame: 'unknown',
  FrameStrata: 'FrameStrata',
  AzeriteEmpoweredItemLocation: 'ItemLocationMixin'
} as const

export const WOW_TYPE_KEYS = Object.keys(WowTypeKeyAliases)
