const [_Tinkr] = [...$vararg]
const Tinkr = _Tinkr as unknown as tinkr

interface PrintArgs {
  text: string
  prefix?: string
  level?: 'info' | 'warn' | 'error'
  color?: string
}

function Print(this: void, { text, prefix, level, color }: PrintArgs) {
  let textColor: string | undefined = color

  if (!textColor) {
    textColor =
      level === 'error'
        ? 'FF76CE'
        : level === 'info'
        ? '94FFD8'
        : level === 'warn'
        ? 'FDFFC2'
        : 'A3D8FF'
  }

  print(`|cFF${textColor}[${prefix || 'Mekanome'}]|r |cFFFFFFFF ${text}`)
}

function GetObject(this: void, token: string): WowGameObject | false {
  return (Object as typeof TObject)(token)
}

function Error(this: void, text: string) {
  Print({ text, prefix: 'ERROR', level: 'error' })
  error(text)
}

const Mekanome: Partial<IMekanome> = {
  enabled: false,
  Print,
  Error,
  GetObject
}

function Load(this: void, path: string) {
  return Tinkr.Util.Evaluator.LoadString(
    Tinkr.Util.File.Read(path),
    [Tinkr, Mekanome],
    path
  ).default
}

function LoadDirectoryFiles(this: void, path: string) {
  const directories = ListDirectories(path)

  for (const directory of directories) {
    LoadDirectoryFiles(`${path}/${directory}`)
  }

  const files = ListFiles(path)

  for (const file of files) {
    Load(`${path}/${file}`)
    Print({ text: `${path}/${file}`, prefix: 'Loaded' })
  }
}

function Init(this: void) {
  Print({ text: '(V) (°,,,,°) (V)' })

  LoadDirectoryFiles('scripts/utils')
  LoadDirectoryFiles('scripts/managers')
  LoadDirectoryFiles('scripts/actions')

  // @ts-ignore
  const keys: (keyof IMekanome)[] = Object.keys(Mekanome)

  keys.forEach((key) => {
    const module = Mekanome[key]
    if (
      module !== undefined &&
      typeof module === 'object' &&
      // @ts-ignore
      module['Init'] !== undefined
    ) {
      // @ts-ignore
      module.Init()

      Print({ text: key, prefix: 'Initialized', level: 'info' })
    }
  })
}

Init()

export default {}
