const [_, _Mekanome] = [...$vararg]
const Mekanome = _Mekanome as unknown as IMekanome

function New(this: void, token: string): Auras {
  const auras = new LuaMap<number, AuraData>()

  const helpfulSlots = C_UnitAuras.GetAuraSlots(
    token as UnitToken,
    'HELPFUL',
    200,
    undefined
  )

  helpfulSlots.forEach((s) => {
    if (s !== undefined) {
      const aura = C_UnitAuras.GetAuraDataBySlot(token, s)
      if (aura) {
        auras.set(aura.auraInstanceID, aura)
      }
    }
  })

  const harmfulSlots = C_UnitAuras.GetAuraSlots(
    token as UnitToken,
    'HARMFUL',
    200,
    undefined
  )

  harmfulSlots.forEach((s) => {
    if (s !== undefined) {
      const aura = C_UnitAuras.GetAuraDataBySlot(token, s)
      if (aura) {
        auras.set(aura.auraInstanceID, aura)
      }
    }
  })

  return auras
}

function GetUpdatedAuras(this: void, updateInfo: UnitAuraUpdateInfo, mob: Mob): Auras {
  const auras: Auras = new LuaMap()

  // If its a full update, just return a fresh aura manager.
  if (updateInfo.isFullUpdate) {
    return New(mob.token)
  }

  if (mob.auras !== undefined) {
    for (const [instanceId, aura] of mob.auras) {
      auras.set(instanceId, aura)
    }
  }

  // Remove any removed auras from the table.
  if (updateInfo.removedAuraInstanceIDs !== undefined) {
    updateInfo.removedAuraInstanceIDs.forEach((id) => {
      if (auras.has(id)) {
        auras.delete(id)
      }
    })
  }
  // Add any added auras to the table.
  if (updateInfo.addedAuras !== undefined) {
    updateInfo.addedAuras.forEach((data) => {
      auras.set(data.auraInstanceID, data)
    })
  }

  if (updateInfo.updatedAuraInstanceIDs) {
    updateInfo.updatedAuraInstanceIDs.forEach((id) => {
      const data = C_UnitAuras.GetAuraDataByAuraInstanceID(mob.token, id)
      if (data) {
        auras.set(data.auraInstanceID, data)
      }
    })
  }

  return auras
}

const _AuraManager = {
  New,
  GetUpdatedAuras
}

declare global {
  type Auras = LuaMap<number, AuraData>
  type AuraManager = typeof _AuraManager
  interface Aura {
    id: number
    isPlayerAura?: boolean
  }
}

Mekanome.AuraManager = _AuraManager

export default {}
