const [_, _Mekanome] = [...$vararg]
const Mekanome = _Mekanome as unknown as IMekanome

let isMoving = false
let isCasting = false
let isChanneling = false
let isAlive = true
let isMounted = false
let Player: Mob | undefined = undefined
let Target: Mob | undefined = undefined
let Mouseover: Mob | undefined = undefined
const sqw = C_CVar.GetCVar('SpellQueueWindow')
let spellQueueWindow = sqw !== undefined ? Number(sqw) : 0

/**
 * Refresh the players state
 */
function Refresh(this: void) {
  const { MobUtils, MobManager } = Mekanome

  const player = MobManager.GetMob({ token: 'player' })
  const target = MobManager.GetMob({ token: 'target' })
  const mouseover = MobManager.GetMob({ token: 'mouseover' })

  if (player === undefined) {
    throw Mekanome.Error('Unable to refresh player state as player object doesnt exist.')
  }

  Player = player
  Target = target
  Mouseover = mouseover
  isMoving = MobUtils.GetIsMoving(player)
  isMounted = MobUtils.IsMounted(player)
  isAlive = MobUtils.IsAlive(player)

  const castEnd = MobUtils.GetCastEndTime(player)
  const channelEnd = MobUtils.GetChannelEndTime(player)

  if (castEnd) {
    const timeUntilEnd = castEnd - GetTime()
    isCasting = timeUntilEnd > spellQueueWindow / 1000
  } else {
    isCasting = false
  }

  if (channelEnd) {
    const timeUntilEnd = channelEnd - GetTime()
    isChanneling = timeUntilEnd > spellQueueWindow / 1000
  } else {
    isChanneling = false
  }
}

/**
 * Get the players state
 */
function GetPlayerState(this: void) {
  if (Player === undefined) {
    throw Mekanome.Error('Unable to get player state as player object doesnt exist.')
  }

  return {
    Player,
    Target,
    Mouseover,
    spellQueueWindow,
    isAlive,
    isMounted,
    isMoving,
    isCasting,
    isChanneling
  }
}

function Init(this: void) {
  // Listens to CVAR updates to make sure that our local sqw var
  // is kept in sync with the players
  Mekanome.EventManager.RegisterEventHandler<'CVAR_UPDATE'>(
    'CVAR_UPDATE',
    ([eventName]) => {
      if (eventName === 'SpellQueueWindow') {
        const newSQW = C_CVar.GetCVar('SpellQueueWindow')
        if (newSQW !== undefined) {
          spellQueueWindow = Number(newSQW)
        }
      }
    },
    'tm_cvar_update'
  )
}

const _PlayerStateManager = {
  Refresh,
  GetPlayerState,
  Init
}

declare global {
  type PlayerStateManager = typeof _PlayerStateManager
}

Mekanome.PlayerStateManager = _PlayerStateManager

export default {}
