const [_, _Mekanome] = [...$vararg]
const Mekanome = _Mekanome as unknown as IMekanome

/**
 * Creates a useable aura
 */
function Create(this: void, id: number, isPlayerAura?: boolean): Aura {
  return {
    id,
    isPlayerAura
  }
}

/**
 * Returns whether or not an aura is active in a list of auras
 */
function IsUp(this: void, aura: Aura, mob: Mob, secondsToIgnore?: number): boolean {
  let isUp = false
  for (const [, auraData] of mob.auras) {
    const matchesPlayerFilter =
      aura.isPlayerAura === undefined || auraData.sourceUnit === 'player'

    if (auraData.spellId === aura.id && matchesPlayerFilter) {
      if (auraData.expirationTime === 0) {
        isUp = true
      } else {
        const expirationTime = auraData.expirationTime - GetTime()
        if (secondsToIgnore !== undefined) {
          isUp = expirationTime > secondsToIgnore
        } else {
          isUp = expirationTime > 0
        }
      }
      break
    }
  }

  return isUp
}

const _AuraUtils = {
  IsUp,
  Create
}

declare global {
  type AuraUtils = typeof _AuraUtils
}

Mekanome.AuraUtils = _AuraUtils

export {}
