const [_, _Mekanome] = [...$vararg]
const Mekanome = _Mekanome as unknown as IMekanome

/**
 * Casts the provided castable. Can be used for spells, items, macros etc as long as there
 * is a corresponding binding and target / position info.
 */
function Cast(this: void, castable: Castable) {
  const mouseover = Mekanome.GetObject('mouseover')

  function ExecuteBinding(this: void) {
    print(castable.binding)
    RunBinding(castable.binding)
    RunBinding(castable.binding, 'up')
  }
  function ClearMouseover(this: void) {
    SetMouseover(undefined)
  }
  function RestoreMouseover(this: void) {
    if (mouseover !== false) {
      SetMouseover(mouseover)
    }
  }
  /**
   * Will execute the binding on the player, or active 'target'.
   */
  function TargetOrSelfCast(this: void) {
    // Because we have mouseover casting enabled, we need to clear the mouseover before casting.
    ClearMouseover()
    // Then we execute the binding.
    ExecuteBinding()
    // FInally we restore the mouseover to what it was before the binding was executed.
    RestoreMouseover()
  }

  // Casts as if no target was selected.
  if (castable.variant === 'self') {
    TargetOrSelfCast()
  }
  if (castable.variant === 'ground') {
    const position = castable.position

    // First we execute the binding to get the target reticle active.
    ExecuteBinding()

    const pending = IsSpellPending()

    // Then we check to make sure the spell is pending, and if it is we click on the specefied position to cast it
    if (pending === 64) {
      Click(position.x, position.y, position.z)
    }
  }
  // Casts the spell at a target.
  if (castable.variant === 'target') {
    const target = Mekanome.GetObject('target')
    const targetGuid = target !== false ? ObjectGUID(target) : undefined

    const castTarget = castable.target

    if (targetGuid === undefined || castTarget.guid !== targetGuid) {
      const targetObj = Mekanome.GetObject(castTarget.token)
      if (targetObj !== false) {
        SetMouseover(targetObj)
        ExecuteBinding()
        RestoreMouseover()
      }
    } else {
      TargetOrSelfCast()
    }
  }
}

Mekanome.Cast = Cast

export default {}
